define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/aspect",
	"dojo/store/Memory",
	"dojo/store/Observable",
	"dijit/form/Button",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin"
], function(declare, lang, arrayUtils, aspect, Memory, Observable, Button,
			_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin) {

	/*
		Author: wamesbury
		Created: 2020-04-03
		Purpose: A collection of editors of the same type
		History:

		Wishlist:
		- Dynamic actions
		- Maximum editor limit
		- Editor reordering/sorting
		- Ensure uniqueness of value(s) in sub-editors

		Usage notes:
		- Remove action: collection.actionStore.remove("add");
	*/

	var EditorCollection = declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
		actionStore: {},
		editorStore: {},
		ObjectEditor: null,	// thornberry/object/ObjectEditor
		Object: null,		// thornberry/object/Object
		readOnly: false,
		titleText: null,
		disabled: false,
		modified: false,
		uniqueFields: [],
		_disableUpdateUnique: false,

		templateString: '<div data-dojo-attach-point="containerNode">' +
							'<div data-dojo-attach-point="titleNode"></div>'+
							'<div data-dojo-attach-point="editorContainer"></div>'+
							'<div data-dojo-attach-point="actionsContainer"></div>'+
						'</div>',

		constructor: function(data) {
			var t = this;
			lang.mixin(t,data);

			t.set("editorStore", new Observable(new Memory()));
			t.set("actionStore", new Observable(new Memory()));
		},

		buildRendering: function() {
			var t = this;
			t.inherited(arguments);

			if (t.titleText) {
				t.titleNode.innerHTML = t.titleText;
			}

			t.own(t.get("editorStore").query().observe(function(editor, removedFrom, insertedInto) {
				if (removedFrom > -1) {
					editor.destroyRecursive();
				} else if (insertedInto > -1) {
					t.editorContainer.appendChild(editor.domNode);
				}
			}, true));

			t.own(t.get("actionStore").query().observe(function(action, removedFrom, insertedInto) {
				if (removedFrom > -1) {
					action.destroyRecursive();
				} else if (insertedInto > -1) {
					t.actionsContainer.appendChild(action.domNode);
				}
			}, true));

			t.actionButtons();
		},

		add: function(value) {
			var t = this;
			var newEditor = new t.ObjectEditor({ collection: t });
			newEditor.set("object", new t.Object(value));
			
			if (value && value.id) {
				var existingItems = t.get("editorStore").query({id: value.id});
				if (existingItems.length > 0) {
					existingItems[0].set("object", new t.Object(value));
				} else {
					t.get("editorStore").put(newEditor, {id: value.id});
				}
			} else {
				t.get("editorStore").put(newEditor);
			}
			
			t.updateUniqueFields();

			newEditor.own(aspect.after(newEditor,"_modifyValue",function() {
				t.updateUniqueFields();
			}));

			return newEditor;
		},

		remove: function(editorId) {
			var t = this;
			t.get("editorStore").remove(editorId);
			t.set("modified", true);

			t.updateUniqueFields();
		},

		removeAll: function() {
			var t = this;
			
			t._disableUpdateUnique = true;
			t.get("editorStore").query().forEach(function(editor) {
				t.get("editorStore").remove(editor.id);
			});
			t._disableUpdateUnique = false;
			t.set("modified", true);
		},

		isValid: function() {
			var t = this;

			var valid = true;
			t.get("editorStore").query().forEach(function(editor) {
				if (editor.isValid) {
					valid = valid && editor.isValid();
				}
			});

			return valid;
		},

		validate: function() {
			var t = this;

			var valid = true;
			t.get("editorStore").query().forEach(function(editor) {
				if (editor.validate) {
					valid = valid && editor.validate();
				}
			});

			return valid;
		},

		actionButtons: function() {
			var t = this;
			
			var add = new Button({
				label: "Add",
				disabled: t.disabled,
				onClick: function() {
					t.add();
					t.set("modified", true);
				}
			});

			t.get("actionStore").put(add, {id: "add"});
		},

		okToSave: function() {
			/*var t = this;

			var okToSave = true;
			t.get("editorStore").query().forEach(function(editor) {
				if (editor.okToSave) {
					okToSave = editor.okToSave();
				}
			});

			return okToSave;*/
		},

		_getModifiedAttr: function() {
			// summary:
			//		Support get('modified')

			var t = this;
			
			// see if editor collection changed
			var modified = t.modified;
			
			if (!modified) {
				// see if editors themselves changed
				t.get("editorStore").query().forEach(function(editor) {
					modified = modified || editor.get("modified");
				});
			}

			return modified;
		},

		_getValueAttr: function() {
			// summary:
			//		Support get('value')

			var t = this;
			var value = [];
			t.get("editorStore").query().forEach(function(editor) {
				value.push(editor.object.getData());
			});

			return value;
		},

		_setValueAttr: function(value) {
			// summary:
			//		Support set('value')
			
			var t = this;
			var values = [];
			if (value instanceof Array) {
				values = value;
			} else if (typeof value === "undefined") {
				values.push({});
			} else if (typeof value === "object") {
				values.push(value);
			}

			t._disableUpdateUnique = true;
			arrayUtils.forEach(values, function(value) {
				t.add(value);
			});
			t._disableUpdateUnique = false;
		},

		_setDisabledAttr: function(disabled) {
			var t = this;
			disabled = (typeof disabled === "undefined" ? true : disabled);
			t.disabled = disabled;

			// disable editors
			t.get("editorStore").query().forEach(function(editor) {
				editor.set("disabled", disabled);
			});

			// disable action buttons
			t.get("actionStore").query().forEach(function(action) {
				action.set("disabled", disabled);
			});
		},

		_setActionTypeAttr: function(value) {
			/*
				value = "Edit", "Add", or "View"
			*/

			var t = this;
			t.get("editorStore").query().forEach(function(editor) {
				editor.set("actionType", value);
			});

			// disable action buttons
			t.get("actionStore").query().forEach(function(action) {
				action.set("disabled", value == "View");
			});
		},

		_setRequiredAttr: function(required) {
			var t = this;
			required = (typeof required === "undefined" ? true : required);
			t.required = required;

			// require editors
			t.get("editorStore").query().forEach(function(editor) {
				editor.set("required", required);
			});
		},

		updateUniqueFields: function() {
			/*
				Assumes editors are displayed in order created, and that earlier editors take precedence over later
				editors. Will need to change if editor sorting/reordering is implemented.
				
				Supports Select and DateTextBox widgets.

				TODO:
				- Custom field filtering/table query
				- Additional widget types
			*/
			
			var t = this;

			if (!t._disableUpdateUnique && t.uniqueFields.length > 0) {
				var selectedFieldValues = {};

				// Build index of used unique field values
				t.get("editorStore").query().forEach(function(editor) {
					arrayUtils.forEach(t.uniqueFields, function(uniqueField) {
						if (typeof selectedFieldValues[uniqueField] == "undefined") {
							selectedFieldValues[uniqueField] = [];
						}
						selectedFieldValues[uniqueField].push(editor.object.get(uniqueField));
					});
				});

				// Restrict selection
				t.get("editorStore").query().forEach(function(editor) {
					arrayUtils.forEach(t.uniqueFields, function(uniqueField) {
						var widget = editor[uniqueField + "Field"];
						var selectedValue = editor.object.get(uniqueField);
						var propertyModel = editor.getPropertyModel(uniqueField);
						
						if (widget.declaredClass === "dijit.form.Select" || widget.declaredClass == "dijit.form.FilteringSelect") {
							var restrictValueArray = [];

							propertyModel.table.query().forEach(function(tableValue) {
								if ((arrayUtils.indexOf(selectedFieldValues[uniqueField], tableValue.id) === -1) ||
									(tableValue.id == selectedValue)) {
									restrictValueArray.push(tableValue.id);
								}
							});

							if (widget.declaredClass === "dijit.form.Select") {
								editor.populateSelect(widget, propertyModel.table, selectedValue, true, restrictValueArray);
							} else if (widget.declaredClass == "dijit.form.FilteringSelect") {
								editor.populateDndSelect(widget, propertyModel.table, selectedValue, restrictValueArray);
							}
						} else if (widget.declaredClass === "dijit.form.DateTextBox") {
							// Widget requires a custom rangeCheck() implementation; see ADMN:TINS for example
							var constraints = widget.get("constraints");

							var invalidDates = arrayUtils.filter(selectedFieldValues[uniqueField], function(date) {
								return date != null && (selectedValue == null || date.getTime() != selectedValue.getTime());
							});
							constraints.invalidDates = invalidDates;
							widget.set("constraints", constraints);
						}
					});
				});
			}
		}
	});

	return EditorCollection;
});