define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/currency",
    "dojo/number",
	"dojo/date",
    "dojo/date/locale"
],function(declare,lang,currency,dnumber,date,locale) {
	var formatter = declare(null, {
        DEFAULTMAXLENGTH: 100,

        getForColumn: function(column,columnDefinition) {
            /*
                Summary:
                    Returns a function to format the column's data in a grid
                Inputs:
                    column: the column item from the grid structure variable
                    columnDefinition: the data for the column from the thornberry/report/Report object
                Output:
                    a formatter function
            */
            var t = this;
            var formatterFunc = null;
            if (t[columnDefinition.formatter]) {
                formatterFunc = lang.hitch(column,function(itemData) {
                    var displayValue = "";
                    try {
                        displayValue = t[columnDefinition.formatter](itemData[this.field],itemData,columnDefinition,this.field);
                    } catch(exception) {
                        displayValue = "ERROR: unable to format";
                    }
                    return displayValue;
                });
            } else {
                formatterFunc = function() {return "ERROR: formatter not found";};
            }
            return formatterFunc;
        },

        defaultFormatter: function(fieldData,itemData,columnDefinition) {
            /*
                Summary:
                    Returns the column value escaped and truncated to DEFAULTMAXLENGTH
            */
            var t = this;
            fieldData = (typeof fieldData === "undefined" ? "" : fieldData).toString();
			var regex = /\n/g;
            if (fieldData.length > t.DEFAULTMAXLENGTH) {
                fieldData =  "<div class='tooltip-opener-node' data-tooltip-text=\""+escapeText(escapeText(fieldData))+"\">"+escapeText(fieldData.substring(0,t.DEFAULTMAXLENGTH - 3)).replace(regex, '<br/>') + "...</div>";
            } else {
                fieldData = escapeText(fieldData).replace(regex, '<br/>');
            }
            return fieldData;
        },

        patientSummaryLink: function(fieldData,itemData,columnDefinition) {
            /*
                Summary:
                    If the row data includes the NDocNumber property, then returns the column data as a link to the patient summary
            */
            var t = this;
            var result = t.defaultFormatter(fieldData,itemData);
            if (itemData.NDocNumber) {
                result = "<a onclick=\"var topWin = (startWindow && startWindow.OpenSummary ? startWindow : top); topWin.OpenSummary('"+itemData.NDocNumber+"');window.focusOpen('','STARTWINDOW').focus()\">"+result+"</a>"; 
            }
            return result;   
        },

        contactSummaryLink: function(fieldData,itemData,columnDefinition) {
            /*
                Summary:
                    If the row data includes the ContactNDocNumber property, then returns the column data as a link to the patient summary
            */
            var t = this;
            var result = t.defaultFormatter(fieldData,itemData);
            if (itemData.ContactNDocNumber) {
                result = "<a onclick=\"var topWin = (startWindow && startWindow.OpenSummary ? startWindow : top); topWin.OpenSummary('"+itemData.ContactNDocNumber+"');window.focusOpen('','STARTWINDOW').focus()\">"+result+"</a>"; 
            }
            return result;   
        },

        dollars: function(fieldData,itemData,columnDefinition) {
            /*
                Summary:
                    format as $X,XXX.XX
            */
            var t = this;
            var result = t.defaultFormatter(fieldData,itemData);
            if (result !== "") {
                result = "<div style='text-align:right'>"+currency.format(result,{currency:"USD"})+"</div>";
            }
            return result;   
        },

        integer: function(fieldData,itemData,columnDefinition) {
            /*
                Summary:
                    format as an integer
            */
            var t = this;
            var result = t.defaultFormatter(fieldData,itemData);
            if (result !== "") {
                result = dnumber.round(result,0);
                if (isNaN(result)) {
                    result = t.defaultFormatter(fieldData,itemData);
                }
            }
            return result;   
        },
		
        twodecimals: function(fieldData,itemData,columnDefinition) {
            /*
                Summary:
                    format as a double with up two decimal places
            */
            var t = this;
            var result = t.defaultFormatter(fieldData,itemData);
            if (result !== "") {
				result = parseFloat(result)
                result = dnumber.round((result +.00001) * 100)/100;
                if (isNaN(result)) {
                    result = t.defaultFormatter(fieldData,itemData);
                }
            }
            return result;   
        },

        boldIfPositive: function(fieldData,itemData,columnDefinition) {
            var t = this;
            var result = t.defaultFormatter(fieldData, itemData);
            if (result !== "" && result > 0) {
                result = "<strong>" + result + "</strong>";
            }
            return result;
        },

        detailFormatter: function(fieldData,itemData,columnDefinition) {
            /*
                Summary:
                    Returns the column value escaped and truncated to DEFAULTMAXLENGTH
            */
            var t = this;
            fieldData = (typeof fieldData === "undefined" ? "" : fieldData).toString();
            if (fieldData !== "") {
                var summary = "";
                if (itemData["summary"] || itemData["DetailSummary"]) {
                    summary = itemData["summary"] || itemData["DetailSummary"];
                }
                fieldData = "<a onClick='showReportDetails(\""+escapeText(fieldData)+"\",\""+columnDefinition.name+"\",\""+itemData.id+"\",\""+escapeText(summary)+"\")'>"+escapeText(fieldData)+"</a>";
            }
            return fieldData;
        },

        dateTime: function (fieldData, itemData, columnDefinition) {
            var t = this;
            var result = t.defaultFormatter(fieldData, itemData);
            if (result !== "") {
				if (result.length <= 10) {
					result = locale.parse(result,  { datePattern: "yyyy-MM-dd", selector: "date" });
					result = locale.format(result, { datePattern: "MM/dd/yyyy", selector: "date" });
				} else {
					result = result.replace(" ", ", "); // locale seems to need a comma when parsing both date and time
					result = locale.parse(result,  { datePattern: "yyyy-MM-dd", timePattern: "HH:mm" });
					result = locale.format(result, { datePattern: "MM/dd/yyyy", timePattern: "HH:mm" });
					result = result.replace(", ", " ");
				}
            }
            return result;
        },

        timestamp: function (fieldData, itemData, columnDefinition) {
            var t = this;
            fieldData = t.defaultFormatter(fieldData, itemData);
            var result = "";
            if (fieldData !== "") {
                fieldData = fieldData.replace(" ", ", "); // locale seems to need a comma when parsing both date and time

                var fieldPieces = fieldData.split(".");
                var hasMilliseconds = false;
                if (fieldPieces.length > 1) {
                    // We have milliseconds, so make sure it's exactly 3 characters long
                    hasMilliseconds = true;
                    var milliseconds = fieldPieces[1];
                    if (milliseconds.length < 3) {
                        while (milliseconds.length < 3) {
                            milliseconds += "0";
                        }
                    } else if (milliseconds.length > 3) {
                        milliseconds = milliseconds.substring(0,3);
                    }
                    fieldData = fieldPieces[0] + "." + milliseconds;
                }

                if (hasMilliseconds) {
                    result = locale.parse(fieldData, { datePattern: "yyyy-MM-dd", timePattern: "HH:mm:ss.SSS" });
                    result = locale.format(result, { datePattern: "MM/dd/yyyy", timePattern: "HH:mm:ss.SSS" });
                } else {
                    result = locale.parse(fieldData, { datePattern: "yyyy-MM-dd", timePattern: "HH:mm:ss" });
                    result = locale.format(result, { datePattern: "MM/dd/yyyy", timePattern: "HH:mm:ss" });
                }
                result = result.replace(", ", " ");
            }
            return result;
        },

        crossTabCell: function (fieldData, itemData, columnDefinition,field) {
            var t = this;
            var result = t[columnDefinition.originalFormatter](fieldData, itemData, columnDefinition);
            result = "<a onClick='showReportDetails(\""+itemData._row+"\",\""+field+"\")'>"+result+"</a>";
            if (field === "[ALL]") {
                result = "<span class='bold'>"+result+"</span>";
            }
            return result;
        },

        booleanFormatter: function (fieldData, itemData, columnDefinition) {
            var result = "";
            if (typeof fieldData !== "undefined" && fieldData !== "") {
                result = (fieldData == 1 ? "Yes" : "No");
            }
            return result;
        },

        fileLink: function (fieldData, itemData, columnDefinition) {
            // Assumes that the field data is the filename
            var result = "";
            if (typeof fieldData !== "undefined" && fieldData !== "") {
                debugger;
                result = "<a onClick='openFile(\""+escapeText(fieldData)+"\")'>Open File</a>";
            }
            return result;
        },

        minutesToDuration: function (fieldData, itemData, columnDefinition) {
            var t = this;
            var result = t.integer(fieldData, itemData);
            if (result !== "" && (/^\d+$/).test(result)) {

                var hours = Math.floor(result / 60);
                while (hours.toString().length < 2) {
                    hours = "0" + hours;
                }
                
                var minutes = result%60;
                while (minutes.toString().length < 2) {
                    minutes = "0" + minutes;
                }

                result = hours + ":" + minutes;
            }
            return result;
        },
		
        commaFormat: function (fieldData, itemData, columnDefinition) {
            /*
                Summary:
                   Returns code that seperates a comma delimited list into individual items, then writes each of them followed by a line break
            */
			var formatResult = "";
            var result = fieldData.toString();
			var resultList = result.split(",")
			for (var i = 0, len = resultList.length; i < len; i++) {
			  formatResult = formatResult + resultList[i]+ "<br/>";
			}
            return formatResult;
        },
		
        telehealthAlert: function(fieldData,itemData,columnDefinition) {
            var t = this;
            var result = t.defaultFormatter(fieldData,itemData);
            var alertColor = (result.toString().indexOf("above") === -1 ? "blue" : "red");
            result = "<span style='color:"+alertColor+"'>"+result+"</span>"; 
            return result;   
        },
		
        documentDates: function(fieldData,itemData,columnDefinition) {
			var documentDates = JSON.parse(itemData.DocumentDates);
			var dates = "";
			for (var i = 0; i < documentDates.length; i++) {
				dates += "<tr><td class='gridLabel'>"+escapeText(documentDates[i].name) + "</td><td class='gridData'>" + documentDates[i].value + "</td></tr>";
			}
			return (dates === "" ? "" : "<table class='gridTable'>"+dates+"</table>");
        },
		
        documentDueDates: function(fieldData,itemData,columnDefinition) {
			var documentDueDates = JSON.parse(itemData.DocumentDueDates);
			documentDueDates.sort(function(a,b){return a.dueDateInternal - b.dueDateInternal;});
			var dueDateDisplay = "";
			for (var i = 0; i < documentDueDates.length; i++) {
				var dateVal = documentDueDates[i];
				if (dateVal.dueDate !== "") {
					if (dueDateDisplay !== "") {
						dueDateDisplay += "<br>";
					}
					var dateString = escapeText(dateVal.name) + " due by " + dateVal.dueDate;
					if (dateVal.dueStatus === "Overdue") {
						dateString = "<span class='overdue'>" + dateString + "</span>";
						var daysAgo = date.difference(locale.parse(dateVal.dueDate, {selector: "date", datePattern:"MM/dd/yyyy"}), new Date(), "day") - 1;
						if (daysAgo == 1) {
							dateString += " (" + daysAgo + " day ago)";
						} else {
							dateString += " (" + daysAgo + " days ago)";
						}
					} else if (dateVal.dueStatus === "Completed Late") {
						dateString += " (L)";
					} else if (dateVal.dueStatus === "Completed") {
						dateString += " (C)";
					}
					dueDateDisplay += dateString;
				}
			}
			return dueDateDisplay;
        }
	});
	
	return new formatter();
});